'use strict';

/*
 * 筛选栏-复选框组件
 * 动态响应用户在界面上选中的项，给选中项对应的字段赋值
 */
var React = require('react');

var _require = require('yylib-ui'),
    YYClass = _require.YYClass,
    YYCheckbox = _require.YYCheckbox,
    YYCol = _require.YYCol,
    YYButton = _require.YYButton;

var _require2 = require('lodash'),
    isFunction = _require2.isFunction;

var YYFilterMultiple = YYClass.create({
    getDefaultProps: function getDefaultProps() {
        return {
            onSubmit: null, //点击确定的回调方法,传递被勾选的项目集合
            onCancel: null, //点击取消的回调方法
            rowKey: '', //行标识键值
            items: [], //{value:1, text:''} 数组 value值不可重复
            colSpan: 3 //每个单位所占比例，24等份
        };
    },
    getInitialState: function getInitialState() {
        var checkedItems = [];
        var _props = this.props,
            defaultSelectedKeys = _props.defaultSelectedKeys,
            items = _props.items;

        _.forEach(items, function (item) {
            if (defaultSelectedKeys.indexOf(item.value) !== -1) {
                checkedItems.push(item);
            }
        });

        return {
            checkedItems: checkedItems //存储被选择的选项
        };
    },
    _onChange: function _onChange(e) {
        var checkedItems = this.state.checkedItems;
        if (e.target.checked) {
            var checkedItem = { value: e.target.currValue, text: e.target.text };
            checkedItems.push(checkedItem);
            this.setState({ checkedItems: checkedItems });
        } else {
            for (var i = 0; i < checkedItems.length; i++) {
                if (e.target.currValue == checkedItems[i].value) {
                    checkedItems.splice(i, 1);
                    this.setState({ checkedItems: checkedItems });
                    return;
                };
            }
        }
    },
    _onClickSubmit: function _onClickSubmit() {
        var onSubmit = this.props.onSubmit;
        if (isFunction(onSubmit)) {
            onSubmit(this.state.checkedItems, this.props.rowKey);
        }
    },
    _onClickCancel: function _onClickCancel() {
        this.setState({ checkedItems: [] });
        var onCancel = this.props.onCancel;
        if (isFunction(onCancel)) {
            onCancel(this.props.rowKey);
        }
    },
    render: function render() {
        var items = [];
        var checkedItems = this.state.checkedItems;
        var item;
        var isChecked;
        for (var i = 0; i < this.props.items.length; i++) {
            item = this.props.items[i];
            for (var index in checkedItems) {
                if (checkedItems[index].value == item.value) {
                    isChecked = true;
                    break;
                } else {
                    isChecked = false;
                }
            }
            items.push(React.createElement(
                YYCol,
                { span: this.props.colSpan, title: item.text },
                React.createElement(
                    'label',
                    { style: { cursor: 'pointer' } },
                    React.createElement(YYCheckbox, { checked: isChecked, currValue: item.value, text: item.text, onChange: this._onChange, disabled: item.disabled ? item.disabled : false }),
                    item.text
                )
            ));
        }
        return React.createElement(
            'div',
            { className: 'yyui-filter-multiple' },
            React.createElement(
                'div',
                { className: 'ant-row yyui-filter-items' },
                items
            ),
            React.createElement(
                'div',
                { className: 'ant-row yyui-filter-buttons' },
                React.createElement(
                    YYButton,
                    { type: 'primary', onClick: this._onClickSubmit },
                    '\u63D0\u4EA4'
                ),
                React.createElement(
                    YYButton,
                    { style: { marginLeft: '10px' }, type: 'ghost', onClick: this._onClickCancel },
                    '\u53D6\u6D88'
                )
            )
        );
    }
});

module.exports = YYFilterMultiple;