'use strict';

var React = require('react');

var _require = require('yylib-ui'),
    YYClass = _require.YYClass,
    YYToolbar = _require.YYToolbar,
    YYButton = _require.YYButton,
    YYBreadcrumb = _require.YYBreadcrumb,
    YYInputButton = _require.YYInputButton;

var PropTypes = YYClass.PropTypes;
var field = YYClass.field;
var uiMeta = {
    name: 'ijz-header',
    fields: [
    //field参数列表：字段属性名|属性默认值|属性类型|是否必填|属性描述
    field('headerCaption', null, PropTypes.string, false, '主标题'), field('breadcrumbItems', null, PropTypes.array, false, '面包屑内容项'), field('searchPlaceholder', null, PropTypes.string, false, '搜索框placeholder'), field('searchButtonText', '搜索', PropTypes.string, false, '搜索框按钮名称'), field('onSearch', null, PropTypes.function, false, '搜索回调事件'), field('onAddClick', null, PropTypes.function, false, '新增按钮点击事件'), field('addButtonText', '新增', PropTypes.function, false, '新增按钮的文本名称'), field('addButtonVisible', true, PropTypes.bool, false, '新增按钮的是否显示'), field('searchText', '', PropTypes.string, false, '搜索框默认输入内容'), field('addButtonIsGhost', false, PropTypes.bool, false, '新增按钮是否透明显示'), field('onQuoteClick', null, PropTypes.function, false, '引用按钮点击事件'), field('quoteButtonText', '引用', PropTypes.function, false, '引用按钮的文本名称'), field('quoteButtonVisible', false, PropTypes.bool, false, '引用按钮的是否显示'), field('quoteButtonIsGhost', false, PropTypes.bool, false, '引用按钮是否透明显示')]
};

var IjzHeader = YYClass.create({
    uiMeta: uiMeta,
    getInitialState: function getInitialState() {
        return {
            searchText: this.props.searchText //搜索框输入内容
        };
    },
    _onSearch: function _onSearch() {
        if (typeof this.props.onSearch === 'function') {
            this.props.onSearch(this.state.searchText);
        }
    },
    _onChange: function _onChange(e, value) {
        var _this = this;

        //if (e.type !== 'change') return; //只处理OnChange类型事件
        this.setState({
            searchText: value
        }, function () {
            if (value === '') {
                _this._onSearch();
            }
        });
    },
    _onDragStart: function _onDragStart(e) {
        e.dataTransfer.effectAllowed = 'none';
        e.dataTransfer.dropEffect = 'none';
    },
    render: function render() {
        return React.createElement(
            YYToolbar,
            { className: this.className(), enableAffix: 'true' },
            React.createElement(
                'div',
                { className: 'ijz-left-bar' },
                React.createElement(
                    'span',
                    { className: 'ijz-caption' },
                    this.props.headerCaption
                ),
                React.createElement(YYBreadcrumb, { itemsData: this.props.breadcrumbItems })
            ),
            React.createElement(YYInputButton, { className: 'ijz-input-search', type: 'error', placeholder: this.props.searchPlaceholder, value: this.state.searchText,
                onChange: this._onChange, onPressEnter: this._onSearch, onDragStart: this._onDragStart,
                buttonClick: this._onSearch, buttonText: this.props.searchButtonText }),
            React.createElement(
                YYButton,
                { visible: this.props.addButtonVisible, size: 'large', type: 'error', ghost: this.props.addButtonIsGhost,
                    onClick: this.props.onAddClick },
                this.props.addButtonText
            ),
            React.createElement(
                YYButton,
                { visible: this.props.quoteButtonVisible, size: 'large', type: 'error', ghost: this.props.quoteButtonIsGhost,
                    onClick: this.props.onQuoteClick },
                this.props.quoteButtonText
            )
        );
    }
});
module.exports = IjzHeader;