'use strict';

var Cookies = require('js-cookie');
var CryptoJS = require('./crypto-js');
var Ajax = require('./ajax');
var Cipher = require('./Cipher').default;

var _require = require('./Cipher'),
    opaqueDecrypt = _require.opaqueDecrypt;

var Gzip = require('./Gzip').default;

var transType = 'NCC';
var gzipFlag = true;

/**
 * 为全局添加业务信息
 * 如： 业务日期 业务集团信息 用户idsuperSign
 * @param {String} businessDate 业务日期
 * @param {String} userId 用户id
 * @param {String} groupId 集团id
 */
function getBusinessInfo() {
    if (window.top && typeof window.top.GETBUSINESSINFO === 'function') {
        return window.top.GETBUSINESSINFO();
    } else {
        return null;
    }
}

/**
 * 加签处理方法
 * @param data
 * @param userCode
 * @param allin
 * @returns {*}
 */
function addDataSign(data, userCode) {
    var allin = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

    if (typeof transSn === "undefined" || typeof transSign === "undefined" || !userCode) {
        return data;
    }
    // 先加签全部数据 或者部分数据
    var dataStr = JSON.stringify(data);
    // 这里会影响原字符串 （编码、位数？）导致后台解析不了字符串
    var strDataSign = (allin ? dataStr : data.busiParamJson) + "";
    // 加签 -- bbqin
    data.sysParamJson.sn = transSn(userCode);
    data.sysParamJson.signdata = transSign(strDataSign, userCode);
    return data;
}

function transSn(usercode) {
    var result = '';
    if (transType == 'NCC') {
        result = usercode;
    } else {
        //ca根据type判断，type为本文件配置
        throw new Error('需要CA认证，但是未安装相应代码');
    }
    return result;
}

//传输签名一定是不需要密码的
function transSign(value, uid) {
    var result = '';
    if (transType == 'NCC') {
        result = CryptoJS.enc.Base64.stringify(CryptoJS.SHA256(uid + value));
    } else {
        //ca根据type判断，type为本文件配置
        throw new Error('需要CA认证，但是未安装相应代码');
    }
    return result;
}

function noop() {}

/**
 *
 * @param url
 * @param data = {busiParamJson: {}, sysParamJson: {}}
 * @param success
 * @param error
 * @param complete
 */
function httpRequestNcc(url) {
    var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var success = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : noop;
    var error = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : noop;
    var complete = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : noop;

    var gzipTools = new Gzip();
    var cipherFlag = localStorage.getItem('rockin');
    var userInfos = getBusinessInfo();
    var userCode = userInfos && userInfos.userCode ? userInfos.userCode : null;
    var _data = {};
    var aeskey = '';
    var cckks = Cookies.get("cookiets") + '';
    aeskey = opaqueDecrypt(localStorage.getItem('cowboy'));
    aeskey = cckks + aeskey.substring(0, aeskey.length - cckks.length);
    Cipher.CipherFlag = cipherFlag === "true";

    if (data.sysParamJson === undefined) {
        _data['sysParamJson'] = {};
    } else {
        _data['sysParamJson'] = data.sysParamJson;
    }
    if (data.busiParamJson === undefined) {
        _data['busiParamJson'] = {};
    } else {
        _data['busiParamJson'] = JSON.stringify(data.busiParamJson);
    }

    var newData = addDataSign(_data, userCode);

    newData.sysParamJson['ts'] = new Date().getTime();
    newData.sysParamJson['pagecs'] = cckks;

    newData = function (data) {
        var gData = '';
        if (gzipFlag) {
            gData = gzipTools.zip(JSON.stringify(data));
        } else {
            gData = JSON.stringify(data);
        }
        return Cipher.encrypt(gData, aeskey);
    }(newData);

    Ajax.postJSONEx(url, newData, function (result) {
        var gData = '';
        var data = Cipher.decrypt(result, aeskey);
        if (gzipFlag) {
            gData = gzipTools.unzip(JSON.stringify(data));
        } else {
            gData = JSON.stringify(data);
        }
        success(gData);
    }, error, complete);
}

module.exports.default = { httpRequestNcc: httpRequestNcc, getBusinessInfo: getBusinessInfo };