"use strict";

var _ = require('lodash');
var React = require('react');
var addEventListener = require('add-dom-event-listener');
var YYClass = require('../../base/YYClass');
var YYInput = require('../input/YYInput');
var TriggerMenu = require('./TriggerMenu');
var EntryRefDataModal = require('./EntryRefDataModal');
var BaseHost = require('../../../business/BaseHost');

var _require = require('../../refer/constants/ReferConstants'),
    GET_REFINFO_BYCODE = _require.GET_REFINFO_BYCODE;

require('./style/index');

var _require2 = require('./utils'),
    checkValueTypeAndValue = _require2.checkValueTypeAndValue,
    getValuePropValue = _require2.getValuePropValue,
    getPropValue = _require2.getPropValue,
    getObjectValuePropValue = _require2.getObjectValuePropValue,
    generateMenuItem = _require2.generateMenuItem,
    getDataByReferCode = _require2.getDataByReferCode,
    getSearchParams = _require2.getSearchParams,
    requestSearchData = _require2.requestSearchData;

var field = YYClass.field,
    PropTypes = YYClass.PropTypes;


function noop() {}

var uiMeta = {
    name: 'autoselect',
    fields: [
    /* field 参数列表 */
    /**** 字段属性名：属性默认值：属性类型：是否必填：属性描述 ****/
    field('serverUrl', BaseHost, PropTypes.string, true, '参照服务的地址信息(域名和端口)'), field('apiUrl', GET_REFINFO_BYCODE, PropTypes.string, true, '获取参照注册信息的api相对地址，注意：域名和端口信息需要写在serverUrl'), field('value', undefined, PropTypes.oneOfType([PropTypes.object, PropTypes.string]), false, '默认初始值'), //,ok
    field('disabled', false, PropTypes.bool, false, '禁用'), //ok
    field('maxLength', undefined, PropTypes.oneOf([PropTypes.string, PropTypes.number]), false, '可输入字符长度'), field('onChange', noop, PropTypes.func, false, '确认操作回调函数'), //ok
    field('onSelect', noop, PropTypes.func, false, '被选中时调用，参数为选中项的 value 值'), //ok
    field('onClearClick', noop, PropTypes.func, false, '文本框清除后的回调'), //ok
    field('size', 'default', PropTypes.oneOf(['large', 'default', 'small']), false, '选择框大小，可选 large small'), //ok
    field('placeholder', '输入关键词匹配', PropTypes.string, false, '选择框默认文字'), //ok
    field('style', null, PropTypes.object, false, '样式'), //ok
    field('maxSize', "20", PropTypes.string, false, '下拉子选项最大显示条数'), //ok
    // field('propKey', null, PropTypes.oneOf([PropTypes.string, PropTypes.array]), false, '需要加到对象value里的其它属性'),//#
    field('onDealData', null, PropTypes.func, false, '查询数据之后的渲染下拉回调'), //参照原始返回值
    // field('valueKey', null, PropTypes.string, false, '指定下拉选项的value取值'),//#
    // field('textKey', null, PropTypes.string, false, '指定下拉选项的text取值'),//#
    field('onSearchParams', null, PropTypes.func, false, '需要自定义传递参数时使用此回调'), //ok
    field('autoSearch', null, PropTypes.bool, false, '当根据searchUrl查找数据时，是否初始化就查询'), //ok
    field('autoOne', null, PropTypes.bool, false, '当autoSearch为true时生效，当查到的数据只有一条时自动显示在输入框中'), //ok
    field('refreshTime', 0, PropTypes.number, false, '下拉选项刷新，通过refreshTime参数传时间戳(new Date().getTime())来刷新参照数据，供应用代码动态变更属性来完成刷新调用'), //ok
    field('refinfokey', null, PropTypes.string, true, '参照code'), //ok
    field('condition', null, PropTypes.oneOfType([PropTypes.func, PropTypes.object]), false, '查询条件'), //URL查询query参数//ok
    field('valueType', 'string', PropTypes.string, false, '值类型string, object'), //ok
    field('renderItemText', null, PropTypes.func, true, '自定义渲染匹配值的下拉项的显示内容文本，传递参数：option-当前下拉项数据对象、label-当前下拉显示的文本'), field('onCreateNew', null, PropTypes.func, true, '匹配值不存在的情况下，显示新增项，新增项被点击时的回调'), field('renderNewText', null, PropTypes.func, true, '匹配值不存在的情况下，自定义显示新增项的文本'), field('onSubmitSuccess', noop, PropTypes.func, false, '新建新的选项提交成功的回调'), field('onSubmitFail', noop, PropTypes.func, false, '新建新的选项提交失败的回调')]
};

var YYAutoSelect = YYClass.create({
    uiMeta: uiMeta,

    /**
     * 参照接口方法
     */
    mixins: [{ getDataByReferCode: getDataByReferCode, getSearchParams: getSearchParams, requestSearchData: requestSearchData }],
    /**
     * 参照接口方法
     */

    getInitialState: function getInitialState() {
        var props = this.props;
        // checkValueTypeAndValue(props); //暂时关闭类型检查

        var value = null;
        if ('value' in props) value = props.value;
        var inputValue = '';
        inputValue = value ? _.isObject(value) ? value.name : value : '';

        var options = []; //菜单子项

        var open = false; //下拉展开开关
        var focus = false; //输入框焦点
        var modalVisible = false; //录入参照数据窗口
        var autoSearch = false; //初始自动查询

        /** 业务状态**/
        var dataUrl = null; //请求数据url
        var createUrl = null; //创建数据url
        var keyField = {
            idField: '', //id关键字
            codeField: '', //编码关键字
            nameField: '' //名称关键字
        };
        /** 业务状态**/

        return {
            autoSearch: autoSearch,
            inputValue: inputValue,
            focus: focus,
            open: open,
            dataUrl: dataUrl,
            createUrl: createUrl,
            keyField: keyField,
            options: options,
            modalVisible: modalVisible
        };
    },
    componentWillMount: function componentWillMount() {
        var _props = this.props,
            autoSearch = _props.autoSearch,
            refinfokey = _props.refinfokey,
            serverUrl = _props.serverUrl,
            apiUrl = _props.apiUrl,
            id = _props.id;

        this.state.autoSearch = autoSearch;
        this.getDataByReferCode(refinfokey, serverUrl + apiUrl, id);
    },
    componentDidMount: function componentDidMount() {
        var parentDom = document.querySelector('.yyui-app');
        if (parentDom) {
            this.handleScrollListener = addEventListener(parentDom, 'scroll', this.onOuterBlur);
        }
    },
    componentWillUnMount: function componentWillUnMount() {
        if (this.handleScrollListener) {
            this.handleScrollListener.remove();
        }
    },
    componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
        var _props2 = this.props,
            refinfokey = _props2.refinfokey,
            refreshTime = _props2.refreshTime,
            value = _props2.value;
        var keyField = this.state.keyField;

        if (nextProps && nextProps.refinfokey && nextProps.refinfokey != refinfokey) {
            this.getDataByReferCode(nextProps.refinfokey, nextProps.serverUrl + nextProps.apiUrl);
        }
        if (nextProps && nextProps.refreshTime !== undefined && refreshTime !== nextProps.refreshTime) {
            this.setState({ options: [] });
        }
        if (value !== nextProps.value) {
            this.state.inputValue = nextProps.value ? _.isObject(nextProps.value) ? nextProps.value[keyField.nameField] || nextProps.value['name'] : nextProps.value : '';
        }
    },
    onInputChange: function onInputChange(event) {
        var _this = this;

        var props = this.props;
        var state = this.state;
        var val = event.target.value.replace(/[\r\n]/g, '');
        clearTimeout(this.searchTime);
        this.searchTime = setTimeout(function () {
            _this.requestSearchData(val, state.dataUrl, _this.getSearchParams, false, props.id);
        }, 500);
        props.onChange(val, {}, false);
        this.setState({ inputValue: val, focus: true });
    },
    onMenuSelect: function onMenuSelect(_ref) {
        var item = _ref.item;

        var options = this.state.options;
        var props = this.props;
        var state = this.state;
        // const selectedValue = getValuePropValue(item);
        // const selectedLabel = getPropValue(item);
        var selectedLabel = item.props[state.keyField.nameField];
        clearTimeout(this.searchTime);
        state.focus = false;
        if (item.props.newAddRefer) {
            if (_.isFunction(props.onCreateNew)) {
                this.setState({ open: false });
                return props.onCreateNew(item.props.searchText);
            } else {
                return this.setState({ modalVisible: true }); //打开添加参照数据窗口
            }
        }
        var event = selectedLabel;
        if (props.valueType === 'object') event = getObjectValuePropValue(item, this.state.keyField);
        props.onSelect(event, item.props, item);
        props.onChange(event, item.props, props.autoOne !== null && options.length === 1);
        var inputValue = void 0;
        inputValue = selectedLabel;
        this.setState({ inputValue: inputValue, open: false });
    },
    onOuterBlur: function onOuterBlur() {
        this.setState({ open: false, focus: false });
    },
    onInputClick: function onInputClick() {
        var options = this.state.options;

        if (options.length) this.setState({ open: true, focus: true });
    },
    onInputClear: function onInputClear() {
        this.state.inputValue = '';
        this.state.options = [];
        if (typeof this.props.onClearClick == 'function') {
            this.props.onClearClick();
        }
    },


    //对话框取消回调
    onModalVisible: function onModalVisible(e) {
        this.setState({ modalVisible: false });
    },


    //对话框确定回调
    onModalOk: function onModalOk() {
        this.setState({ modalVisible: false });
    },


    /**
     * 生成菜单项
     */
    generateMenuItem: generateMenuItem,

    render: function render() {
        var _props3 = this.props,
            disabled = _props3.disabled,
            maxSize = _props3.maxSize,
            renderItemText = _props3.renderItemText,
            placeholder = _props3.placeholder,
            size = _props3.size,
            style = _props3.style,
            onSubmitSuccess = _props3.onSubmitSuccess,
            onSubmitFail = _props3.onSubmitFail,
            maxLength = _props3.maxLength;
        var _state = this.state,
            inputValue = _state.inputValue,
            open = _state.open,
            options = _state.options,
            modalVisible = _state.modalVisible;


        return React.createElement(
            'div',
            { className: this.className(), style: style },
            React.createElement(
                TriggerMenu,
                {
                    options: this.generateMenuItem(options, renderItemText),
                    style: style,
                    visible: open,
                    onMenuSelect: this.onMenuSelect
                },
                React.createElement(YYInput, {
                    disabled: disabled,
                    maxLength: maxLength,
                    placeholder: placeholder,
                    value: inputValue,
                    onBlur: this.onOuterBlur,
                    onChange: this.onInputChange,
                    onClick: this.onInputClick,
                    onClearClick: this.onInputClear
                })
            ),
            React.createElement(EntryRefDataModal, {
                disabled: disabled,
                size: size,
                visible: modalVisible,
                onOk: this.onModalOk,
                onCancel: this.onModalVisible,
                onSubmitSuccess: onSubmitSuccess,
                onSubmitFail: onSubmitFail
            })
        );
    }
});

module.exports = YYAutoSelect;